[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Classifications client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.1-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.Classifications)](https://pypi.org/project/fds.sdk.Classifications/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

The Classifications API gives access to various Industry Classifications for a given list of securities. Beginning first with GICS Direct, the service will then expand to offering NAICS, SIC, RBICS, and more. 


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.1
- SDK version: 1.0.12
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.Classifications==1.0.12
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.Classifications==1.0.12
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.Classifications
from fds.sdk.Classifications.api import gics_api
from fds.sdk.Classifications.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.Classifications.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.Classifications.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.Classifications.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = gics_api.GICSApi(api_client)
    ids = [
        "ids_example",
    ] # [str] | The requested list of security identifiers. Accepted ID types include Market Tickers, SEDOL, ISINs, CUSIPs, or FactSet Permanent Ids. <p>***ids limit** =  1000 per request*</p> *<p>Make note, GET Method URL request lines are also limited to a total length of 8192 bytes (8KB). In cases where the service allows for thousands of ids, which may lead to exceeding this request line limit of 8KB, its advised for any requests with large request lines to be requested through the respective \"POST\" method.</p>* 
    start_date = "startDate_example" # str | Requested start date expressed in YYYY-MM-DD format. (optional)
    end_date = "endDate_example" # str | Requested End Date for Range expressed in YYYY-MM-DD format. (optional)
    frequency = "M" # str | Controls the display frequency of the data returned.   * **D** = Daily   * **W** = Weekly, based on the last day of the week of the start date.   * **M** = Monthly, based on the last trading day of the month.   * **AM** = Monthly, based on the start date (e.g., if the start date is June 16, data is displayed for June 16, May 16, April 16 etc.).   * **CQ** = Quarterly based on the last trading day of the calendar quarter (March, June, September, or December).   * **AY** = Actual Annual, based on the start date.   * **CY** = Calendar Annual, based on the last trading day of the calendar year.  (optional) if omitted the server will use the default value of "M"
    calendar = "FIVEDAY" # str | Calendar of data returned. SEVENDAY includes weekends. (optional) if omitted the server will use the default value of "FIVEDAY"

    try:
        # Gets the GICS Direct Classifications
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_gics(ids, start_date=start_date, end_date=end_date, frequency=frequency, calendar=calendar)

        pprint(api_response)
    except fds.sdk.Classifications.ApiException as e:
        print("Exception when calling GICSApi->get_gics: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Gets the GICS Direct Classifications
    #     api_response, http_status_code, response_headers = api_instance.get_gics_with_http_info(ids, start_date=start_date, end_date=end_date, frequency=frequency, calendar=calendar)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Classifications.ApiException as e:
    #     print("Exception when calling GICSApi->get_gics: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Gets the GICS Direct Classifications
    #     async_result = api_instance.get_gics_async(ids, start_date=start_date, end_date=end_date, frequency=frequency, calendar=calendar)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.Classifications.ApiException as e:
    #     print("Exception when calling GICSApi->get_gics: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Gets the GICS Direct Classifications
    #     async_result = api_instance.get_gics_with_http_info_async(ids, start_date=start_date, end_date=end_date, frequency=frequency, calendar=calendar)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Classifications.ApiException as e:
    #     print("Exception when calling GICSApi->get_gics: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.Classifications

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.Classifications.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.Classifications

configuration = fds.sdk.Classifications.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.Classifications

configuration = fds.sdk.Classifications.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.Classifications

configuration = fds.sdk.Classifications.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*GICSApi* | [**get_gics**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/GICSApi.md#get_gics) | **GET** /classifications/v1/gics | Gets the GICS Direct Classifications
*GICSApi* | [**post_gics**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/GICSApi.md#post_gics) | **POST** /classifications/v1/gics | Returns the GICS classifications for the requested &#39;ids&#39; and date range.


## Documentation For Models

 - [Calendar](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/Calendar.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/ErrorResponse.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/ErrorResponseSubErrors.md)
 - [Frequency](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/Frequency.md)
 - [Gics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/Gics.md)
 - [GicsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/GicsRequest.md)
 - [GicsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/GicsResponse.md)
 - [Ids](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Classifications/v1/docs/Ids.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.Classifications.apis and fds.sdk.Classifications.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.Classifications.api.default_api import DefaultApi`
- `from fds.sdk.Classifications.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.Classifications
from fds.sdk.Classifications.apis import *
from fds.sdk.Classifications.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

